from fooddatacentral import nutrients
import pandas as pd 
import unittest

apikey = "kY45fKdbAFHas9GpxBKlDyEYbwvfC00z17oKd3ba"

class TestNutrients(unittest.TestCase):
    def test_column(self):
        """
        Test that it returns a df with the desirable columns
        """
        #171705 = Avocados, raw, all commercial varieties
        expected = ['Quantity', 'Units']
        actual = nutrients(apikey,171705)
        self.assertEqual(expected, actual.columns.tolist())	
    
    def test_reasonable(self):
        """
        Test that it returns a df with reasonable nutrient quantities and units
        """
        #171705 = Avocados, raw, all commercial varieties
        result  = nutrients(apikey,171705)
        actual_q = result.loc['Energy', 'Quantity']
        actual_u = result.loc['Energy', 'Units']
        expected_u = ['kJ', 'kiloJoule ', 'kj', 'kilojoule ']
        self.assertTrue(0 <= actual_q<= 1000)	
        self.assertTrue(actual_u in expected_u)	

if __name__ == '__main__':
    unittest.main()



from urllib.request import Request, urlopen
import pandas as pd
import json
from json import JSONDecodeError
import warnings
import os
import numpy as np
import requests
from functools import lru_cache
from pint import UnitRegistry, UndefinedUnitError, DimensionalityError

ureg = UnitRegistry()

home = os.path.expanduser('~')
unitrc = ('unitsrc',os.path.join(home,'.unitsrc'))

for fn in unitrc:
    try:
        ureg.load_definitions(fn)
        break
    except (IOError, ValueError):
        pass


@lru_cache(maxsize=256)
def search(apikey, term, url = 'https://api.nal.usda.gov/fdc/v1/search',num_results=50):
    """
    Search Food Central Database, using apikey and "term" as search criterion.
    
    Parameters
    ----------
    apikey : str
        Your personal API key
    term : str, int
        A string of the food name, or a specific FDC ID in int format
    url : str
        default = 'https://api.nal.usda.gov/fdc/v1/search'
    pageSize : int
        Number of results to return per page.  Default is 50.

    Returns
    -------
        a pd.DataFrame of matches.
    """
    parms = (('format', 'json'),('generalSearchInput', term),('api_key', apikey),('pageSize',num_results))
    r = requests.get(url, params = parms)
    if r.status_code == 200:
        if 'foods' in r.json():
            l = r.json()['foods']
        else:
            warnings.warn("No foods found for '%s'." % term)
            return []
    elif r.status_code == 404:
        warnings.warn("Couldn't find %s." % term)
        return []
    else:
        warnings.warn("Some problem with request: %s" % str(r))
        return []

    return pd.DataFrame(l)

@lru_cache(maxsize=256)
def nutrients(apikey, fdc_id, url = 'https://api.nal.usda.gov/fdc/v1/food/',verbose=False):
    """
    Construct a food report for food with given fdc_id; nutrients are given per 100 g or 100 ml of the food.
    
    Parameters
    ----------
    apikey : str
        Your personal API key
    fdc_id : int
        a specific FDC ID in int format
    url : str
        default = 'https://api.nal.usda.gov/fdc/v1/search'

    Returns
    -------
        a pd.DataFrame of nutrients and quantities.
    """
    params = (('api_key', apikey),)
    try:
        r = requests.get(url+"%s" % fdc_id, params = params)

        if r.status_code == 200:
            L = r.json()['foodNutrients']
        elif r.status_code == 404:
            warnings.warn("Couldn't find fdc_id=%s." % fdc_id)
            return None
        else:
            warnings.warn("Some problem with request: %s" % str(r))
            return None
        if verbose: print(r.header)
    except KeyError:
        return None

    v = {}
    u = {}
    for l in L:
        if l['type'] == "FoodNutrient":
            try:
                v[l['nutrient']['name']] = l['amount']  # Quantity
            except KeyError: # No amount?
                v[l['nutrient']['name']] = 0
                
            u[l['nutrient']['name']] = l['nutrient']['unitName']  # Units

    #print(l)
    N = pd.DataFrame({'Quantity':v,'Units':u})

    return N

def units(q,u,ureg=ureg):
    """Convert quantity q of units u to 100g or 100ml."""
    try:
        x = ureg.Quantity(float(q),u)
    except UndefinedUnitError:
        return ureg.Quantity(np.nan,'ml')

    try:
        return x.to(ureg.hectogram)
    except DimensionalityError:
        return x.to(ureg.deciliter)

@lru_cache(maxsize=256)
def ingredients(apikey, fdc_id, url = 'https://api.nal.usda.gov/fdc/v1/food/'):
    """
    Given fdc_id of a Survey Food, return ingredients of food.
    
    Parameters
    ----------
    apikey : str
        Your personal API key
    fdc_id : int
        a specific FDC ID in int format; should be a Survey Food
    url : str
        default = 'https://api.nal.usda.gov/fdc/v1/search'

    Returns
    -------
        a pd.DataFrame of ingredients and quantities.
    """
    params = (('api_key', apikey),)
    try:
        r = requests.get(url+"%s" % fdc_id, params = params)

        if r.status_code == 200:
            L = r.json()['inputFoods']
        elif r.status_code == 404:
            warnings.warn("Couldn't find fdc_id=%s." % fdc_id)
        else:
            warnings.warn("Some problem with request: %s" % str(r))
            return None
    except KeyError:
        warnings.warn("Make sure it's a Survey Food (FNDDS); couldn't find fdc_id=%s." % fdc_id)
        return None
    if L == [] and r.json()['dataType']== 'SR Legacy':
        warnings.warn("No ingredients data for this SR Legacy food fdc_id=%s; " % fdc_id)
        return L
    v = []
    c = []
    w = []
    a = []
    for l in L:
        v.append(l['foodDescription'])
        c.append(l['ingredientCode'])
        w.append(l['ingredientWeight'])
        a.append(l['amount'])
    if w != a:
        return pd.DataFrame({'Ingredient':v,'Food Code/NDB Number':c, 'Weight (grams)':w, 'Amount':a})
    return pd.DataFrame({'Ingredient':v,'Food Code/NDB Number':c, 'Weight (grams)':w})



"""Package to obtain information from the USDA's Food Data Central data project.

See https://fdc.nal.usda.gov/api-guide.html for API documentation; see
https://fdc.nal.usda.gov/api-key-signup.html for API key; please also refer to
the documentation provided in the README.md.

Optional txt file of units '.unitsrc' can live in the HOME directory with
specification of necessary unit conversions (e.g., "extra_large_egg = 56 * grams
= xl_egg" and "tea_bag = 3 * grams"). See documentation for the pint package at
https://pint.readthedocs.io/.

Functions:

    search(str, str/int, url) -> pd.DataFrame
    nutrients(str, int, url) -> pd.DataFrame
    units(int/float, str , ureg) -> ureg
    ingredients(str, int, url) -> pd.DataFrame

"""

from .fooddatacentral import *



from fooddatacentral import ingredients
import pandas as pd 
import unittest, warnings


apikey = "kY45fKdbAFHas9GpxBKlDyEYbwvfC00z17oKd3ba"

class TestIngredients(unittest.TestCase):
    def test_warn1(self):
        """
        Test that it returns a warning for branded foods
        """
        #2107531 = CEREAL; Branded food
        #173889 = Cereals ready-to-eat, POST, Honeycomb Cereal; SR legacy food
        warnings.simplefilter("always")
        with warnings.catch_warnings(record=True) as w:
            ingredients(apikey, 2107531)
            self.assertEquals(len(w), 1)
            
    def test_warn2(self):
        """
        Test that it returns an emptylist and a warning for  SR legacy foods
        """
        #173889 = Cereals ready-to-eat, POST, Honeycomb Cereal; SR legacy food
        actual= ingredients(apikey, 173889)
        expect = []
        self.assertEquals(expect, actual)
        
        warnings.simplefilter("always")
        with warnings.catch_warnings(record=True) as w:
            ingredients(apikey, 173889)
            self.assertEquals(len(w), 1)
if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages
from os.path import abspath, dirname, join

README_MD = open(join(dirname(abspath(__file__)), "README.md")).read()

setup(
    name="fooddatacentral",
    version="1.0.12",
    packages=find_packages(exclude="tests"),

    description="Python module to obtain information from the USDA's Food Data Central data project.",
    long_description=README_MD,
    long_description_content_type="text/markdown",

    url="https://github.com/ligon/fooddatacentral",

    author_name="Ethan Ligon",
    author_email="ligon@berkeley.edu",
    license = "Creative Commons Attribution-NonCommercial-ShareAlike 4.0. http://creativecommons.org/licenses/by-nc-sa/4.0/",
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3 :: Only"
    ],

    keywords="USDA, FDC"
)



from fooddatacentral import search
import pandas as pd 
import unittest
from warnings import warn 

apikey = "kY45fKdbAFHas9GpxBKlDyEYbwvfC00z17oKd3ba"

class TestSearch(unittest.TestCase):
    def test_reasonable(self):
        """
        Test that it returns an empty df for an illogical food input
        """
        actual = search(apikey,"avocao")
        warn('Invalid term; check spelling of food names or digits of food code')
        self.assertTrue(actual.empty)	

    def test_string(self):
        """
        Test that it returns a non-empty df for a string input of food name
        """
        expected = 173573
        actual = search(apikey,"avocado")
        lst = actual['fdcId'].tolist()
        self.assertTrue(expected in lst)	
    
    def test_int(self):
        """
        Test that it returns a non-empty df for an integer input of FDC ID
        """
        expected = 'oil, avocado'
        actual = search(apikey,173573)
        lst = actual['lowercaseDescription'].tolist()
        self.assertTrue(expected in lst)	

if __name__ == '__main__':
    unittest.main()
